// ==UserScript==
// @id             baidupan@ywzhaiqi@gmail.com
// @name           BaiduPanDownloadHelper
// @version        2.5
// @namespace      https://github.com/ywzhaiqi
// @author         ywzhaiqi@gmail.com
// @description    批量导出百度盘的下载链接（百度盘的打包不能断点续传且速度慢）
// @grant          GM_setClipboard
// @grant          GM_openInTab
// @grant          GM_xmlhttpRequest
// @grant          GM_addStyle
// @grant          unsafeWindow
// @homepageURL    http://userscripts.org/scripts/show/162138
// @updateURL      http://userscripts.org/scripts/source/162138.meta.js
// @downloadURL    https://userscripts.org/scripts/source/162138.user.js
// @include        *n.baidu.com/share/link*
// @include        *n.baidu.com/s/*
// @run-at         document-end
// ==/UserScript==

// 参考 BaiduPanMD5Button http://userscripts.org/scripts/show/156906
/**
 * 百度盘代码解析，2013/07/22
 *   文件：yun_home_speed_all.js
 *   函数：performSelectionDownload
 */

(function(CSS){

var $ = unsafeWindow.jQuery;

if(typeof $ == "undefined"){
    return;
}

var debug = 0 ? console.debug : function(){};

var FileUtils = unsafeWindow.FileUtils,
    Utilities = unsafeWindow.Utilities,
    disk = unsafeWindow.disk,
    Page = unsafeWindow.Page;

var getListURL = "http://pan.baidu.com/share/list?channel=chunlei&clienttype=0&web=1&page=1&shareid=" +
                  disk.getParam("shareid") + "&uk=" + disk.getParam("uk") + "&dir=";

var ns = {
    checkedItems: [],  // 勾选的条目

    init: function(){
        var ins = $('#file_action_buttons');
        if(ins.length == 0)
            return;

        GM_addStyle(CSS);

        var button = $('<a class="bbtn" style="padding-left:10px"><b>批量下载</b></a>')
                        .appendTo(ins);
        button[0].addEventListener("click", ns.buttonClicked, false);
    },
    buttonClicked: function(event){
        ns.notice = Utilities.useToast({
            toastMode: disk.ui.Toast.MODE_CAUTION,
            msg: "正在获取中，请稍后...",
            sticky: true
        });

        var fetchURL;
        ns.checkedItems = FileUtils.getListViewCheckedItems();
        ns.fetchCount = 0;
        ns.checkedItems.forEach(function(item){
            if(item.isdir){
                ns.fetchCount += 1;
                fetchURL = getListURL + encodeURIComponent(item.path);
                console.log("GM_xmlhttpRequest: " + fetchURL);

                GM_xmlhttpRequest({
                    method: 'GET',
                    url: fetchURL,
                    onload: function(response){
                        var JSONObj = JSON.parse(response.responseText);
                        ns.fetchCount -= 1;
                        if(JSONObj.errno == 0){
                            item.children = JSONObj.list;
                            ns.handleResult();
                        }else{
                            console.log("获取json数据错误: " + item.path);
                        }
                    }
                });
            }
        });

        // 没有文件夹的情况
        ns.handleResult();
    },
    handleResult: function(){
        // 全部获取完成
        if(ns.fetchCount == 0){
            debug("全部获取完成", ns.checkedItems);
            ns.showPanel();
            ns.notice.setVisible(false);
        }
    },
    showPanel: function(){
        if(!ns.panel){
            ns.panel = ns.createPanel();
        }else{
            var linksHTML = ns.createDLinksHtml();
            $("#mDownload-links").html(linksHTML);
            ns.panel.style.display = "block";
        }
    },
    createPanel: function(){
        var container = document.createElement("div");
        container.id = "mDownload-container";

        var links_div = document.createElement("div");
        links_div.id = "mDownload-links";
        links_div.innerHTML = ns.createDLinksHtml();

        var closeButton = document.createElement("button");
        closeButton.id = "mDownload-close-button";
        closeButton.innerHTML = "关闭";
        closeButton.onclick = function(){
            container.style.display = "none";
        };

        var exportButton = document.createElement("button");
        exportButton.id = "mDownload-export-button";
        exportButton.innerHTML = "导出";
        exportButton.onclick = function(){
            GM_openInTab('data:text/html;charset=utf-8,' + encodeURIComponent(links_div.innerHTML));
        };

        var copyButton = document.createElement("button");
        copyButton.id = "mDownload-copy-button";
        copyButton.innerHTML = "复制到剪贴板";
        copyButton.onclick = ns.copyDlinks;

        container.appendChild(closeButton);
        container.appendChild(exportButton);
        container.appendChild(copyButton);
        container.appendChild(links_div);
        document.body.appendChild(container);

        return container;
    },

    dir_tpl: "<b style='padding-left:{padding_left}'>{server_filename}</b>",
    dlinks_tpl: "<a class='dlinks' href={dlink} style='padding-left:{padding_left}'>{server_filename}</a>",
    createDLinksHtml: function(){
        var htmls = [];
        var isAdded = false;
        ns.checkedItems.forEach(function(item){
            item.padding_left = "0px";
            if(item.isdir){
                htmls.push(template(ns.dir_tpl, item));
                if(item.children){
                    item.children.forEach(function(i){
                        i.padding_left = "15px";
                        if(i.dlink){
                            htmls.push(template(ns.dlinks_tpl, i));
                        }else{
                            htmls.push(template(ns.dir_tpl, i));
                        }
                    });
                }
            }else if(item.dlink){
                if(!isAdded){
                    htmls.push("<b>---------------</b>");
                    isAdded = true;
                }
                htmls.push(template(ns.dlinks_tpl, item));
            }
        });
        return  htmls.join("<br>");
    },
    copyDlinks: function(){
        var dlinks = [];
        ns.checkedItems.forEach(function(item){
            if(item.isdir && item.children){
                item.children.forEach(function(i){
                    dlinks.push(i.dlink);
                });
            }else{
                dlinks.push(item.dlink);
            }
        });

        GM_setClipboard(dlinks.join('\n'), 'text');

        Utilities.useToast({
            toastMode: disk.ui.Toast.MODE_CAUTION,
            msg: '已经复制 <b>' + dlinks.length + '</b> 条下载链接到剪贴板',
            sticky: false
        });
    },
};

ns.init();

function template(template, data) {
    return template.replace(/\{([\w\.]*)\}/g, function(str, key) {
        var keys = key.split('.'),
            value = data[keys.shift()];
        keys.forEach(function(key) {
            value = value[key];
        });
        return (value === null || value === undefined) ? '' : value;
    });
}

})('\
#mDownload-container{\
    position: fixed;\
    z-index: 1000;\
    left: 314px;\
    top: 120px;\
    width: 500px;\
    background: white;\
    padding: 10px;\
    border: 1px solid rgb(153, 153, 153);\
    box-shadow: 0px 0px 9px rgb(153, 153, 153);\
}\
#mDownload-links{\
    margin-top: 10px;\
    max-height: 400px;\
    overflow: auto;\
}\
#mDownload-links b{\
    color: red;\
}\
#mDownload-close-button{\
    margin-right: 10px;\
}\
#mDownload-export-button{\
    margin-right: 10px;\
}\
');